# -*- encoding: utf-8 -*-
# kb v0.1.7
# A knowledge base organizer
# Copyright © 2020, gnc.
# See /LICENSE for licensing information.

"""
Main routine of kb.

:Copyright: © 2020, gnc.
:License: GPLv3 (see /LICENSE).
"""
from kb.main import main


__all__ = ('main',)


if __name__ == '__main__':
    main()
