# -*- encoding: utf-8 -*-
# kb v0.1.7
# A knowledge base organizer
# Copyright 2020, Giuseppe Nebbione.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
A knowledge base organizer

:Copyright: © 2020, gnc.
:License: GPLv3 (see /LICENSE).
"""

__title__ = 'kb'
__version__ = '0.1.7'
__author__ = 'gnc'
__license__ = 'GPLv3'
__docformat__ = 'restructuredtext en'

__all__ = ()

# import gettext
# G = gettext.translation('kb', '/usr/share/locale', fallback='C')
# _ = G.gettext
